package edu.sjsu.rhinofaces;

import java.beans.FeatureDescriptor;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.logging.Logger;

import javax.el.ELContext;
import javax.el.ELResolver;
import javax.el.PropertyNotFoundException;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;

/**
 * 
 * @author Tom Austin
 */
public class RhinoELResolver extends ELResolver
{
   private static Logger logger = Logger.global;
   
   public final static String JS_LAST_VAR = "__JS_LAST_VAR";
   private final static String JS_ENVIRONMENT_VARIABLE = "__JS_ENVIRONMENT_VARIABLE";

   /**
      Constructor.
    */
   public RhinoELResolver()
   {
      super();
   }
   
   /**
      Gets the scripting environment for the current session.
    */
   public JavaScriptEnvironment getJavaScriptEnvironment(ELContext context)
   {
      /*VariableMapper vm = context.getVariableMapper();
      
      RhinoValueExpression rhinoValExp = (RhinoValueExpression) vm.resolveVariable(JS_ENVIRONMENT_VARIABLE);
      if (rhinoValExp == null) {
         logger.info("Initializing new JS environmet");
         rhinoValExp = new RhinoValueExpression();
         vm.setVariable(JS_ENVIRONMENT_VARIABLE, rhinoValExp);
      }
      return (JavaScriptEnvironment) rhinoValExp.getValue(context);*/
      
      FacesContext facesContext = (FacesContext)  context.getContext(FacesContext.class);
      ExternalContext externalContext = facesContext.getExternalContext();
      
      Map<String,Object> sessMap = externalContext.getSessionMap();
      
      if (sessMap.containsKey(JS_ENVIRONMENT_VARIABLE))
      {
         return (JavaScriptEnvironment) sessMap.get(JS_ENVIRONMENT_VARIABLE);
      }
      else
      {
         logger.info("Initializing new JS environmet");
         JavaScriptEnvironment jsEnv = new JavaScriptEnvironment();
         sessMap.put(JS_ENVIRONMENT_VARIABLE, jsEnv);
         return jsEnv;
      }
   }

   /**
      @see javax.el.ELResolver#getValue(javax.el.ELContext, java.lang.Object,
           java.lang.Object)
    */
   @Override
   public Object getValue(ELContext context, Object base, Object property)
   {
      logger.info("THA: getValue  " + base + "." + property);
      
      JavaScriptEnvironment jsEnv = getJavaScriptEnvironment(context);
      if (jsEnv == null) return null;
      
      //Property in this case is the variable
      if (base == null)
      {
         if (property == null) throw new PropertyNotFoundException();
         
         if (jsEnv.hasVar(property))
         {
            context.setPropertyResolved(true);
            JSVar jsVar = new JSVar("" + property);
            
            //TODO: Get rid of this... somehow
            jsEnv.setLastVariableCalled(jsVar);
            /////////
            
            return jsVar;
            //return property;
         }
         else return null;
         
      }
      else if (base instanceof JSVar)
      //else if (jsEnvironment.hasVar(base))
      {
         context.setPropertyResolved(true);
         return jsEnv.evalRhino(((JSVar)base).getName() + "." + property + ";");
         //return jsEnv.evalRhino(base + "." + property + ";");
      }
      else return null;
   }

   /**
      @see javax.el.ELResolver#setValue(javax.el.ELContext, java.lang.Object,
           java.lang.Object, java.lang.Object)
    */
   @Override
   public void setValue(ELContext context, Object base, Object property,
         Object value)
   {
      logger.info("THA: setValue  " + base + "." + property + "=" + value);
      
      JavaScriptEnvironment jsEnv = getJavaScriptEnvironment(context);
      jsEnv.evalRhino(((JSVar)base).getName() + "." + property + "='" + value + "'");
      //jsEnv.evalRhino(base + "." + property + "='" + value + "'");
   }
   

   /**
      @see javax.el.ELResolver#getCommonPropertyType(javax.el.ELContext,
           java.lang.Object)
    */
   @Override
   public Class<?> getCommonPropertyType(ELContext arg0, Object arg1)
   {
      logger.info("THA: getCommonPropertyType");
      // TODO Auto-generated method stub
      return null;
   }

   /**
      @see javax.el.ELResolver#getFeatureDescriptors(javax.el.ELContext,
           java.lang.Object)
    */
   @Override
   public Iterator<FeatureDescriptor> getFeatureDescriptors(ELContext arg0,
         Object arg1)
   {
      logger.info("THA: getFeatureDescriptors");
      // TODO Auto-generated method stub
      return null;
   }

   /**
      @see javax.el.ELResolver#getType(javax.el.ELContext, java.lang.Object,
           java.lang.Object)
    */
   @Override
   public Class<?> getType(ELContext arg0, Object arg1, Object arg2)
   {
      logger.info("THA: getType" + " arg1=" + arg1 + " arg2=" + arg2);  
      return null;
   }

   /**
      @see javax.el.ELResolver#isReadOnly(javax.el.ELContext, java.lang.Object,
           java.lang.Object)
    */
   @Override
   public boolean isReadOnly(ELContext arg0, Object arg1, Object arg2)
   {
      logger.info("THA: isReadOnly");
      // TODO Auto-generated method stub
      return false;
   }
}
